package com.nhbc.pda.data.serial;

import java.io.InputStream;
import java.io.ObjectInputStream;
import java.text.DecimalFormat;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import com.nhbc.pda.depthCalc.Constants;
import com.nhbc.pda.data.serial.*;

/**
 * A class for accessing the tree calculator data.
 *
 */
public class DepthData
{
  private Hashtable lut;   // Hashtable to store data from data file
  
  /**
  * Return an String array containing a list a volumn change entries
  * used in a dropdown or list of values.
  * @return           String array
  */
  public String[] getVolumePotentialList()
  {
    Vector vecKeys = loadVectorData(Constants.FILE_VOLUMECHANGE);
    String[] volumnChange = new  String[vecKeys.size()];
    
    AnVolumeChangeEntry volumnEntry;   
    
    // Load values using Key 
    for(int i=0;i<vecKeys.size();i++) 
    { 
      String key = (String)vecKeys.get(i); 
     // String value = lut.get(key).toString(); 
      volumnEntry = (AnVolumeChangeEntry) lut.get (key);
      
      // Add value to string array
      volumnChange[i] = Math.round(Double.parseDouble(volumnEntry.vcp_ModifiedPlasticityIndexMin)) + "% to < " + Math.round(Double.parseDouble(volumnEntry.vcp_ModifiedPlasticityIndexMax)) + "% - " + volumnEntry.vcp_VolumeChangePotential;
    } 
    return volumnChange;
  }
  
   /**
  * Return an String array containing a list a volumn change values for the
  * option selected, used in the final calculation.
  * 
  * @param strVolumePotential lable of select value
  * @return  String[] array containing selected values
  */ 
   public String[] getVolumePotentialValues(String strVolumePotential)
  {
    //if (vecWaterDemand == null)  
    Vector vecKeys = loadVectorData(Constants.FILE_VOLUMECHANGE);
    
    String[] straVolumePotential = new  String[5];
    
    AnVolumeChangeEntry volumnEntry;   
    
    // Load values using Key 
    for(int i=0;i<vecKeys.size();i++) 
    { 
      String key = (String)vecKeys.get(i); 
     // String value = lut.get(key).toString(); 
      volumnEntry = (AnVolumeChangeEntry) lut.get (key);

      if (strVolumePotential.equals(volumnEntry.vcp_VolumeChangePotential))
      {
        // Add value to string array
         straVolumePotential[0] = volumnEntry.vcp_CompressibleMaterialThicknessmm;
         straVolumePotential[1] = volumnEntry.vcp_GroundFloorPrecastConcreteVoidmm;
         straVolumePotential[2] = volumnEntry.vcp_GroundFloorSuspendedTimberVoidmm;
         straVolumePotential[3] = volumnEntry.vcp_MinFoundationDepthOutsideZOIm;
         straVolumePotential[4] = volumnEntry.vcp_VolumeChangePotentialLUID;
      }
    } 
    return straVolumePotential;
  }
  
//   public String[] getFoundationDepthValues(String strFoundationDepth)
//  {
//    //if (vecWaterDemand == null)  
//    Vector vecKeys = loadVectorData(Constants.FILE_FOUDATIONDEPTH);
//    
//    String[] straVolumePotential = new  String[5];
//    
//    AnFoundationDepthEntry foundationDepthEntry;   
//    
//    // Load values using Key 
//    for(int i=0;i<vecKeys.size();i++) 
//    { 
//      String key = (String)vecKeys.get(i); 
//      String value = lut.get(key).toString(); 
//      foundationDepthEntry = (AnFoundationDepthEntry) lut.get (key);
//
//      if (strFoundationDepth.equals(foundationDepthEntry..vcp_VolumeChangePotential))
//      {
//        // Add value to string array
//         straVolumePotential[0] = volumnEntry.vcp_CompressibleMaterialThicknessmm;
//         straVolumePotential[1] = volumnEntry.vcp_GroundFloorPrecastConcreteVoidmm;
//         straVolumePotential[2] = volumnEntry.vcp_GroundFloorSuspendedTimberVoidmm;
//         straVolumePotential[3] = volumnEntry.vcp_MinFoundationDepthOutsideZOIm;
//         straVolumePotential[4] = volumnEntry.vcp_VolumeChangePotentialLUID;
//      }
//    } 
//    return straVolumePotential;
//  }
//  
  
  
  /**
  * Return an String array containing a list of tree species entries
  * used in a dropdown or list of values.
  * @return           String array
  */
  public String[] getTreeSpeciesList()
  {
    Vector vecTrees = loadVectorData(Constants.FILE_TREES);
    String[] straTrees= new  String[vecTrees.size()];
    
    AnTreeEntry treeEntry;   
    
    straTrees[0] = "Unknown";  // add an additional entry
    
    // Load values using Key 
    for(int i=1;i<vecTrees.size();i++) 
    { 
      String key = (String)vecTrees.get(i); 
    //  String value = lut.get(key).toString(); 
      treeEntry = (AnTreeEntry) lut.get (key);

      // Add value to string array
      straTrees[i] = treeEntry.treeSpecies;
    } 
    return straTrees;
  }
  
  /**
  * Return an String array containing a list of tree group entries
  * used in a dropdown or list of values.
  * @return  String array
  */
  public String[] getTreeGroupList()
  {
    Vector vecTreeGroup = loadVectorData(Constants.FILE_TREEGROUP);
    String[] straTreeGroup = new  String[vecTreeGroup.size()];
    
    AnTreeGroupEntry treeGroupEntry;   
    
    // Load values using Key 
    for(int i=0;i<vecTreeGroup.size();i++) 
    { 
      String key = (String)vecTreeGroup.get(i); 
   //   String value = lut.get(key).toString(); 
      treeGroupEntry = (AnTreeGroupEntry) lut.get (key);

      // Add value to string array
      straTreeGroup[i] = treeGroupEntry.tg_treeGroupName;
    } 
    return straTreeGroup;
  }
  
  /**
  * Return the id of the chosen tree group.
  * @param strTreeGroup tree group 
  * @return  String id
  */
   public String getTreeGroupID(String strTreeGroup)
  {
    //if (vecTreeGroup == null)      
    
    Vector vecTreeGroup = loadVectorData(Constants.FILE_TREEGROUP);
    
    String strValue = "";
    
    AnTreeGroupEntry treeGroupEntry;   
    
    // Load values using Key 
    for(int i=0;i<vecTreeGroup.size();i++) 
    { 
      String key = (String)vecTreeGroup.get(i); 
    //  String value = lut.get(key).toString(); 
      treeGroupEntry = (AnTreeGroupEntry) lut.get (key);

      if (strTreeGroup.equals(treeGroupEntry.tg_treeGroupName))
      {
        strValue = treeGroupEntry.tg_treeGroupLUID;
      }
    } 
    return strValue;
  }
  
    /**
  * Return an String array containing a list of Wate rDemand entries
  * used in a dropdown or list of values.
  * @return  String array
  */
  public String[] getWaterDemandList()
  {
   Vector vecWaterDemand = loadVectorData(Constants.FILE_WATERDEMAND);
       
    String[] straWaterDemand = new  String[vecWaterDemand.size()];
    
    AnWaterDemandEntry waterEntry;   
    
    // Load values using Key 
    for(int i=0;i<vecWaterDemand.size();i++) 
    { 
      String key = (String)vecWaterDemand.get(i); 
      waterEntry = (AnWaterDemandEntry) lut.get (key);
      // Add value to string array
      straWaterDemand[i] = waterEntry.wd_waterdemand;
    } 
    return straWaterDemand;
  }
  
  /**
  * Return an String array containing a list of Water Demand values.
  * @param strWaterDemand selected water demand value
  * @return  String array
  */
  public String[] getWaterDemandValues(String strWaterDemand)
  {
    Vector vecWaterDemand = loadVectorData(Constants.FILE_WATERDEMAND);
    String[] straWaterDemand = new  String[vecWaterDemand.size()];
    
    AnWaterDemandEntry waterEntry;   
    
    // Load values using Key 
    for(int i=0;i<vecWaterDemand.size();i++) 
    { 
      String key = (String)vecWaterDemand.get(i); 
      waterEntry = (AnWaterDemandEntry) lut.get (key);

      if (strWaterDemand.equals(waterEntry.wd_waterdemand))
      {
        // Add value to string array
        straWaterDemand[0] = waterEntry.wd_zoneOfInfluenceMatureHtFactor;
        straWaterDemand[1] = waterEntry.wd_waterdemandluid;
      }
    } 
    return straWaterDemand;
  }
 
  /**
  * Return an String array containing .....
  * @param strTree selected tree
  * @return  String[] array
  */ 
  public String[][] getAllTreeSpeciesValues()
  {
    String[][] straTreeValues= new String[70][3];

      Vector  vecTrees = loadVectorData(Constants.FILE_TREES);
 
      
      AnTreeEntry treeEntry;   
      
      // Load values using Key 
      for(int i=0;i<vecTrees.size();i++) 
      { 
        String key = (String)vecTrees.get(i); 
      //  String value = lut.get(key).toString(); 
        treeEntry = (AnTreeEntry) lut.get (key);
  
    //    if (strTree.equals(treeEntry.treeSpecies))
    //    {
  
          // Add value to string array
          straTreeValues[i][0] = treeEntry.matureHeight;
          straTreeValues[i][1] = treeEntry.treeGroupID;
          straTreeValues[i][2] = treeEntry.waterDemandID;
     //   }
      
    }
    
    return straTreeValues;
  } 
 
  
   /**
  * Return an String array containing a list of Tree related data.
  * @param strTree selected tree
  * @return  String[] array
  */ 
  public String[] getTreeValues(String strTree)
  {
    String[] straTreeValues= new  String[3];
      
    if (strTree.length() > 0)
    {
      Vector  vecTrees = loadVectorData(Constants.FILE_TREES);
 
      
      AnTreeEntry treeEntry;   
      
      // Load values using Key 
      for(int i=0;i<vecTrees.size();i++) 
      { 
        String key = (String)vecTrees.get(i); 
      //  String value = lut.get(key).toString(); 
        treeEntry = (AnTreeEntry) lut.get (key);
  
        if (strTree.equals(treeEntry.treeSpecies))
        {
  
          // Add value to string array
          straTreeValues[0] = treeEntry.matureHeight;
          straTreeValues[1] = treeEntry.treeGroupID;
          straTreeValues[2] = treeEntry.waterDemandID;
        }
      }
    }
    
    return straTreeValues;
  }

  /**
  * Return an String array containing a list of Status.
  *
  * @return  String[] array
  */ 
  public String[] getStatusList()
  {
    String[] straStatus = new  String[3];
    
    straStatus[0] = "Remaining";
    straStatus[1] = "Removed";
    straStatus[2] = "Proposed";

    return straStatus;
  }
  
  /**
  * Return an String array containing a list a climate zones
  * used in a dropdown or list of values.
  * @return  String array
  */
  public String[] getClimateZoneList()
  {
    String[] straClimateZones = new String[11];
    double dblValue = 0.00;
    
    for(int i= 0; i <straClimateZones.length; i++)
    { 
      DecimalFormat df2 = new DecimalFormat( "0.00" );
          
      String strValue = new Double(df2.format(dblValue).toString()).toString();
          
      //when formating 0.1 it does not pad, add additional zero (0.10)
      if (strValue.length() == 3 ) strValue = strValue + "0";
        straClimateZones[i] = strValue;
        dblValue = dblValue +0.05;
     }
     
        // currently not able to use because its not ordered
        
      //    Vector vecKeys = loadVectorData(Constants.FILE_CLIMATEZONE);
      //    String[] climateZone = new String[vecKeys.size()];
      //    
      //    AnClimateZoneEntry climateEntry;    
      //    // Load values using Key 
      //    for(int i=0;i<vecKeys.size();i++) 
      //    { 
      //     String key = (String)vecKeys.get(i); 
      //     String value = lut.get(key).toString(); 
      //     climateEntry = (AnClimateZoneEntry) lut.get (key);
      //     
      //     climateZone[i] = climateEntry.cz_ClimateZoneValue;
      //    } 

    return straClimateZones;
  }
  
  
  /**
  * Return an String array containing a list a Ground Level
  * used in a dropdown or list of values.
  * @return  String array
  */
  public String[] getGroundLevelList()
  {
    String[] straGroundLevel = new String[3];
    
    straGroundLevel[0] = "Unaltered";
    straGroundLevel[1] = "Reduced";
    straGroundLevel[2] = "Increased";
     
    return straGroundLevel;
  }
  
  
//  /**
//  * Return an String array containing a list a ground level options
//  * used in a dropdown or list of values
//  * @return  String array
//  */
//  public String[] getGroundLevelList()
//  {
//    Vector vecKeys = loadVectorData(Constants.FILE_GROUNDLEVEL);
//    String[] groundLevel = new String[vecKeys.size()];
//      
//    AnGroundLevelEntry groundEntry;    
//    // Load values using Key 
//    for(int i=0;i<vecKeys.size();i++) 
//    { 
//      String key = (String)vecKeys.get(i); 
//      String value = lut.get(key).toString(); 
//      groundEntry = (AnGroundLevelEntry) lut.get (key);
//
//      groundLevel[i] = groundEntry.gl_name;
//    } 
//    
//   return groundLevel;
//  }
   
  /**
  * Return the help text for a component.
  * @param strComponentName component name to link to help text
  * @return  String 
  */
  public String[] getHelpText(String strComponentName)
  {
    // Only load if the vector is empty
    Vector vecKeysHelpText = loadVectorData(Constants.FILE_HELPTEXT);
  
    String[] helpText= new String[2];  // create new string array to hold help text and title
    
    // an entry to read the data from the hashtable  
    AnHelpTextEntry helpEntry;    
    
    // Load values using Key 
    for(int i=0;i<vecKeysHelpText.size();i++) 
    { 
      String key = (String)vecKeysHelpText.get(i); 
   //   String value = lut.get(key).toString(); 
      helpEntry = (AnHelpTextEntry) lut.get (key);
      
      if (strComponentName.equals(helpEntry.helpTargetName.toString()))
      {
        helpText[0] = helpEntry.helpText.toString(); 
        helpText[1] = helpEntry.helpTitle.toString(); 
       
        return helpText ;    
      }
    } 
    return helpText;
  }
   
  /**
  * Loads the values from a serialized file into a vector.
  * @param fileName name of file to load
  * @return  Vector
  */ 
  public Vector loadVectorData(String fileName)
  {
    lut = null;
    Vector vecKeys = new Vector();
     
    try
    {
      InputStream is = this.getClass().getResourceAsStream(fileName);
      ObjectInputStream ois = new ObjectInputStream (is);
      lut = (Hashtable) ois.readObject();
      ois.close();
     
      //Gets keys from hashtable 
      Enumeration enum = lut.keys(); 
      
      // loop and add to vector
      while (enum.hasMoreElements()) 
      { 
        vecKeys.add(enum.nextElement()); 
      } 
               
      //Sorts vector in Ascending order 
      Collections.sort(vecKeys); 
      
    } catch (Exception ex) {
     System.out.println("Error in loadVectorData: " + ex.getMessage());
     ex.printStackTrace();
    } 
    return vecKeys;
  }

  /**
   * Return the hashtable used.
   * @return  Hashtable
   */ 
  public Hashtable getHashTable()
  {
    return lut;
  }
}